// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/RivetAIDA.hh"
#include "Rivet/Tools/Logging.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/ChargedFinalState.hh"
#include "Rivet/Projections/FastJets.hh"
#include "Rivet/Particle.hh"
#include "Rivet/Tools/BinnedHistogram.hh"
/// @todo Include more projections as required, e.g. ChargedFinalState, FastJets, ZFinder...
//

namespace Rivet {

  struct ATLAS_2012_I1125575_profiles {
    int distribution;
    std::string distribution_label;

    AIDA::IProfile1D* _profiles[5][2];
    AIDA::IDataPointSet* _d[5][2];

  };

  struct ATLAS_2012_I1125575_histograms {
    int distribution;
    std::string distribution_label;

    BinnedHistogram<double> _histograms[5][2];

  };

  class ATLAS_2012_I1125575 : public Analysis {
    public:

      /// @name Constructors etc.
      //@{

      /// Constructor
      ATLAS_2012_I1125575()
        : Analysis("ATLAS_2012_I1125575")
      {    }

      //@}


    public:

      /// @name Analysis methods
      //@{

      /// Book histograms and initialise projections before the run
      void init() {

        /// @todo Initialise and register projections here
        const FinalState fs (-4.5, 4.5, 0*GeV);
        addProjection(fs, "FS");

        const ChargedFinalState jet_input(-2.5, 2.5, 0.5*GeV);
        addProjection(jet_input, "JET_INPUT");

        const ChargedFinalState track_input(-1.5, 1.5, 0.5*GeV);
        addProjection(track_input, "TRACK_INPUT");

        const FastJets jets02(jet_input, FastJets::ANTIKT, 0.2);
        addProjection(jets02, "JETS_02");

        const FastJets jets04(jet_input, FastJets::ANTIKT, 0.4);
        addProjection(jets04, "JETS_04");

        const FastJets jets06(jet_input, FastJets::ANTIKT, 0.6);
        addProjection(jets06, "JETS_06");

        const FastJets jets08(jet_input, FastJets::ANTIKT, 0.8);
        addProjection(jets08, "JETS_08");

        const FastJets jets10(jet_input, FastJets::ANTIKT, 1.0);
        addProjection(jets10, "JETS_10");

        m_mNch.distribution = 1;
        m_mNch.distribution_label = "Mean number of tracks";
        InitializeProfiles(m_mNch);

        m_mPtAvg.distribution = 2;
        m_mPtAvg.distribution_label = "Mean of the average track pT in each region";
        InitializeProfiles(m_mPtAvg);

        m_mPtSum.distribution = 3;
        m_mPtSum.distribution_label = "Mean of the scalar sum of track pT in each region";
        InitializeProfiles(m_mPtSum);

        m_Nch.distribution = 4;
        m_Nch.distribution_label = "Distribution of Nch, in bins of leading track-jet pT";
        InitializeHistograms(m_Nch);

        m_PtAvg.distribution = 5;
        m_PtAvg.distribution_label = "Distribution of average track-jet pT, in bins of leading track-jet pT";
        InitializeHistograms(m_PtAvg);

        m_PtSum.distribution = 6;
        m_PtSum.distribution_label = "Distribution of sum of track-jet pT, in bins of leading track-jet pT";
        InitializeHistograms(m_PtSum);

        for(int i=0; i<5; ++i) {
          m_nEvents[i] = 0.0;
        }

      }

      void InitializeProfiles(ATLAS_2012_I1125575_profiles& plots) {
        for(int i=0; i<5; ++i) {
          for(int j=0; j<2; ++j) {
            plots._d[i][j] = bookDataPointSet(plots.distribution, i+1, j+1);

            std::stringstream name;
            name << plots.distribution << i+1 << j+1;
            plots._profiles[i][j] = bookProfile1D(name.str(), binEdges(plots.distribution, i+1, j+1));
          }
        }
      }

      void InitializeHistograms(ATLAS_2012_I1125575_histograms& plots) {
        vector<double> bin_edges = binEdges(m_mNch.distribution, 1, 1);
        for(int i=0; i<5; ++i) {
          for(int y=0; y<2; ++y) {
            for(unsigned int j=0; j<bin_edges.size()-1; ++j) {
              int histogram_number = ((j+1)*2)-((y+1)%2);
              double low_edge = bin_edges[j];
              double high_edge = bin_edges[j+1];
              plots._histograms[i][y].addHistogram(low_edge, high_edge, bookHistogram1D(plots.distribution, i+1, histogram_number));
            }
          }
        }
      }


      /// Perform the per-event analysis
      void analyze(const Event& event) {
        const double weight = event.weight();

        std::vector<Jets*> all_jets;
        Jets jets_02 = applyProjection<FastJets>(event, "JETS_02").jetsByPt(4.0*GeV, MAXDOUBLE, -1.5, 1.5, PSEUDORAPIDITY);
        all_jets.push_back(&jets_02);
        Jets jets_04 = applyProjection<FastJets>(event, "JETS_04").jetsByPt(4.0*GeV, MAXDOUBLE, -1.5, 1.5, PSEUDORAPIDITY);
        all_jets.push_back(&jets_04);
        Jets jets_06 = applyProjection<FastJets>(event, "JETS_06").jetsByPt(4.0*GeV, MAXDOUBLE, -1.5, 1.5, PSEUDORAPIDITY);
        all_jets.push_back(&jets_06);
        Jets jets_08 = applyProjection<FastJets>(event, "JETS_08").jetsByPt(4.0*GeV, MAXDOUBLE, -1.5, 1.5, PSEUDORAPIDITY);
        all_jets.push_back(&jets_08);
        Jets jets_10 = applyProjection<FastJets>(event, "JETS_10").jetsByPt(4.0*GeV, MAXDOUBLE, -1.5, 1.5, PSEUDORAPIDITY);
        all_jets.push_back(&jets_10);

        // Count the number of tracks in the away and transverse regions, for each set of jets
        double _n_ch[5][2] = { {0,0}, {0,0}, {0,0}, {0,0}, {0,0} };
        // Also add up the sum pT
        double _sumpt[5][2] = { {0,0}, {0,0}, {0,0}, {0,0}, {0,0} };
        // ptmean = sumpt / n_ch
        double _ptavg[5][2] = { {0,0}, {0,0}, {0,0}, {0,0}, {0,0} };
        // lead jet pT defines which bin we want to fill
        double _lead_jet_pts[5] = {0.0};

        // Loop over each of the jet radii:
        for(int i=0; i<5; ++i) {
          if(all_jets[i]->size() < 1) continue;

          // Find the lead jet pT and phi
          double lead_jet_pt = all_jets[i]->at(0).momentum().perp();
          _lead_jet_pts[i] = lead_jet_pt;

          double lead_jet_phi = all_jets[i]->at(0).momentum().phi();

          // Loop over each of the charged particles
          const ParticleVector& tracks = applyProjection<ChargedFinalState>(event, "TRACK_INPUT").particlesByPt();
          foreach(const Particle& t, tracks) {

            // Get the delta-phi between the track and the leading jet
            double dphi = mapAngle0ToPi(lead_jet_phi - t.momentum().phi());

            double track_pt = fabs(t.momentum().perp());

            // Find out which region this puts it in.
            // 0 = away region, 1 = transverse region, 2 = toward region
            int region = region_index(dphi);

            // If the track is in the toward region, ignore it.
            if(region == 2) continue;

            // Otherwise, increment the relevant counters
            ++_n_ch[i][region];
            _sumpt[i][region] += track_pt;

          }
          // Calculate the pT_avg for the away and transverse regions.
          // (And make sure we don't try to divide by zero.)
          _ptavg[i][0] = (_n_ch[i][0] == 0 ? 0.0 : _sumpt[i][0] / _n_ch[i][0]);
          _ptavg[i][1] = (_n_ch[i][1] == 0 ? 0.0 : _sumpt[i][1] / _n_ch[i][1]);

          m_nEvents[i] += weight;
        }

        FillProfiles(m_mNch, _n_ch, _lead_jet_pts, weight);
        FillProfiles(m_mPtAvg, _ptavg, _lead_jet_pts, weight);
        FillProfiles(m_mPtSum, _sumpt, _lead_jet_pts, weight);

        FillHistograms(m_Nch, _n_ch, _lead_jet_pts, weight);
        FillHistograms(m_PtAvg, _ptavg, _lead_jet_pts, weight);
        FillHistograms(m_PtSum, _sumpt, _lead_jet_pts, weight);
      }

      void FillProfiles(ATLAS_2012_I1125575_profiles& plots, double _var [5][2], double lead_pt[5], double weight) {
        for(int i=0; i<5; ++i) {
          double pt = lead_pt[i];
          for(int j=0; j<2; ++j) {
            double v = _var[i][j];
            plots._profiles[i][j]->fill(pt, v, weight);
          }
        }
      }

      void FillHistograms(ATLAS_2012_I1125575_histograms& plots, double _var[5][2], double lead_pt[5], double weight) {
        for(int i=0; i<5; ++i) {
          double pt = lead_pt[i];
          for(int j=0; j<2; ++j) {
            double v = _var[i][j];
            plots._histograms[i][j].fill(pt, v, weight);
          }
        }
      }

      inline int region_index(double dphi) {
        assert(inRange(dphi, 0.0, PI, CLOSED, CLOSED));
        if (dphi < PI/3.0) return 2;
        if (dphi < 2*PI/3.0) return 1;
        return 0;
      }


      /// Normalise histograms etc., after the run
      void finalize() {

        double scale = 1.0 / (2*PI);
        FinalizeProfiles(m_mNch, scale);
        FinalizeProfiles(m_mPtSum, scale);
        FinalizeProfiles(m_mPtAvg, 1.0);

        FinalizeHistograms(m_Nch);
        FinalizeHistograms(m_PtAvg);
        FinalizeHistograms(m_PtSum);

      }

      void FinalizeProfiles(ATLAS_2012_I1125575_profiles& plots, double scale) {
        for(int i=0; i<5; ++i) {
          for(int j=0; j<2; ++j) {
            for(int k=0; k<plots._profiles[i][j]->axis().bins(); ++k) {
              double bin_content = plots._profiles[i][j]->binHeight(k);
              double bin_center = (plots._profiles[i][j]->axis().binLowerEdge(k) + plots._profiles[i][j]->axis().binUpperEdge(k))/2.0;

              IDataPoint* currentPoint = plots._d[i][j]->point(k);
              IMeasurement* xCoord = currentPoint->coordinate(0);
              IMeasurement* yCoord = currentPoint->coordinate(1);

              xCoord->setValue(bin_center);
              yCoord->setValue(bin_content*scale);
            }
            delete plots._profiles[i][j];
            //tree().rm(tree().findPath(dynamic_cast<AIDA::IManagedObject&>((plots._profiles[i][j]))));
          }
        }
      }

      void FinalizeHistograms(ATLAS_2012_I1125575_histograms& plots) {
        for(int i=0; i<5; ++i) {
          for(int j=0; j<2; ++j) {
            vector<AIDA::IHistogram1D*> histos = plots._histograms[i][j].getHistograms();
            foreach(AIDA::IHistogram1D* h, histos) {
              scale(h, 1.0/m_nEvents[i]);
            }
          }
        }
      }

      //@}


    private:

      // Data members like post-cuts event weight counters go here
      double m_nEvents[5];

    private:

      ATLAS_2012_I1125575_profiles m_mNch;
      ATLAS_2012_I1125575_profiles m_mPtAvg;
      ATLAS_2012_I1125575_profiles m_mPtSum;

      ATLAS_2012_I1125575_histograms m_Nch;
      ATLAS_2012_I1125575_histograms m_PtAvg;
      ATLAS_2012_I1125575_histograms m_PtSum;

  };



  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(ATLAS_2012_I1125575);

}
